import os
import jwt
import requests
from . import resources

class ClerkClient:
    api_key: str
    public_pem_key: str
    users: resources.UserResource
    base_url: str = "https://api.clerk.com/v1"

    def __init__(self, api_key: str | None=None, public_pem_key: str | None=None) -> None:
        if api_key is None:
            api_key = os.environ.get("CLERK_SECRET_KEY")
        if api_key is None:
            raise Exception(
                "The api_key client option must be set either by passing api_key to the client or by setting the CLERK_SECRET_KEY environment variable"
            )
        self.api_key = api_key
        
        if public_pem_key is None:
            public_pem_key = os.environ.get("CLERK_PEM_PUBLIC_KEY")
        self.public_pem_key = public_pem_key
        
        self.users = resources.UserResource(self)

    
    def decode_jwt(self, token: str):
        if self.public_pem_key:
            return jwt.decode(token, key=self.public_pem_key, algorithms=['RS256', ])
        else: 
            raise Exception(
                "The public_pem_key client option must be set either by passing public_pem_key to the client or by setting the CLERK_PEM_PUBLIC_KEY environment variable"
            )
    
    def get(self,url, params={}):
        res = requests.get(url=f'{self.base_url}/{url}',params=params, headers={
            'Authorization': f'Bearer {self.api_key}'
        })
        print(res.url)
        return res

    def post(self,url, data={}):
        res = requests.post(url=f'{self.base_url}/{url}',data=data, headers={
            'Authorization': f'Bearer {self.api_key}'
        })
        print(res.url)
        return res
    
    def patch(self,url, data={}):
        res = requests.patch(url=f'{self.base_url}/{url}',data=data, headers={
            'Authorization': f'Bearer {self.api_key}'
        })
        print(res.url)
        return res
    
    def delete(self,url):
        res = requests.delete(url=f'{self.base_url}/{url}', headers={
            'Authorization': f'Bearer {self.api_key}'
        })
        print(res.url)
        return res





class UserResource:
    base_url: str = 'users'

    def __init__(self, client) -> None:
        self._client = client
    
    def getUser(self, user_id):
        return self._client.get(f'{self.base_url}/{user_id}')
    
    def getUserList(self, params):
        return self._client.get(f'{self.base_url}', params=params)

    def getCount(self, params):
        return self._client.get(f'{self.base_url}/count', params=params)
    
    def getOrganizationMembershipList(self, user_id, params={}):
        return self._client.get(f'{self.base_url}/{user_id}/organization_memberships', params=params)
    
    def getUserOauthAccessToken(self, user_id, provider):
        return self._client.get(f'{self.base_url}/{user_id}/oauth_access_tokens/{provider}')
    
    def createUser(self, params={}):
        return self._client.post(f'{self.base_url}', data=params)
    
    def verifyPassword(self, user_id, password):
        return self._client.post(f'{self.base_url}/{user_id}/verify_password', data={"password":password})
    
    def banUser(self, user_id):
        return self._client.post(f'{self.base_url}/{user_id}/ban')
    
    def unbanUser(self, user_id):
        return self._client.post(f'{self.base_url}/{user_id}/unban')
    
    def lockUser(self, user_id):
        return self._client.post(f'{self.base_url}/{user_id}/lock')
    
    def unlockUser(self, user_id):
        return self._client.post(f'{self.base_url}/{user_id}/unlock')
    
    def updateUser(self, params={}):
        return self._client.patch(f'{self.base_url}', data=params)
    
    def updateUserMetadata(self,user_id, params={}):
        return self._client.patch(f'{self.base_url}/{user_id}/metadata', data=params)
    
    def deleteUser(self, user_id):
        return self._client.delete(f'{self.base_url}/{user_id}')
    
    def disableUserMFA(self, user_id):
        return self._client.delete(f'{self.base_url}/{user_id}/mfa')


import time
from django.conf import settings
from rest_framework.authentication import get_authorization_header
from rest_framework.exceptions import AuthenticationFailed
from ..client import ClerkClient

class ClerkAuthMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):

        clerk_client = ClerkClient()

        # Get the authentication header from the request
        auth_header = get_authorization_header(request).split()

        user = {}

        # Check if the authentication header is present and valid
        if not auth_header or len(auth_header) != 2:
            user['is_authenticated'] = False
            request.clerk_user = user
            return self.get_response(request)

        # Verify the authentication token with Clerk
        token = auth_header[1].decode()
        decoded_token = clerk_client.decode_jwt(token=token)

        if not decoded_token.get('azp',None) in (settings.ALLOWED_PARTIES or []):
            raise AuthenticationFailed('Unknown source')

        current_unix_time = int(time.time())
        exp_time = decoded_token.get('exp', None)
        nbf_time = decoded_token.get('nbf', None)
        user_id = decoded_token.get('sub', None)
        if exp_time and nbf_time and user_id:
            if current_unix_time > exp_time:
                raise AuthenticationFailed("The token has expired. Login again.")
            if current_unix_time < nbf_time:
                raise AuthenticationFailed('Invalid authentication token')
        else:
            raise AuthenticationFailed('Invalid authentication token')

        user['is_authenticated'] = True
        user['id'] = user_id
        user['decoded_token'] = decoded_token

        if not user:
            raise AuthenticationFailed('Invalid authentication token')

        # Add the user details to the request object
        request.clerk_user = user

        return self.get_response(request)


from rest_framework.permissions import BasePermission

class ClerkAuthenticated(BasePermission):
    """
    Allows access only to authenticated users.
    """

    def has_permission(self, request, view):
        return request.clerk_user.get('is_authenticated',False)